from django.core.management.base import BaseCommand
from django.conf import settings
from django.apps import apps
import os
import sys
import multiprocessing

root = os.getcwd()
django_project = os.path.basename(root)


class Command(BaseCommand):
    help = "Runs this project as a uWSGI application. Requires the uwsgi binary in system path."

    http_port = os.getenv('PORT', '8000')  # for heroku
    socket_addr = None

    def handle(self, *args, **options):
        for arg in args:
            k, v = arg.split('=')
            if k == 'http':
                if self.http_port:
                    self.http_port = v
            elif k == 'socket':
                self.http_port = None
                self.socket_addr = v

        # load the Django WSGI handler
        os.environ.setdefault('UWSGI_MODULE', '%s.wsgi' % django_project)
        # DJANGO settings
        if options['settings']:
            os.environ['DJANGO_SETTINGS_MODULE'] = options['settings']
        else:
            os.environ.setdefault('DJANGO_SETTINGS_MODULE', '%s.settings' % django_project)

        # set protocol as uwsgi
        os.environ.setdefault('UWSGI_PROTOCOL', 'uwsgi')

        # bind the http server to the default port
        if self.http_port:
            os.environ['UWSGI_HTTP_SOCKET'] = ':%s' % self.http_port
        elif self.socket_addr:
            os.environ['UWSGI_UWSGI_SOCKET'] = self.socket_addr
            os.environ.setdefault('UWSGI_CHMOD_SOCKET', '664')

        # by default set 12 workers and <cpu_count> cheaper. cheaper must not
        # exceed workers. user supplied <workers> overrides default.
        worker_count = int(os.environ.get('UWSGI_WORKERS', 12))
        cpu_count = multiprocessing.cpu_count()
        os.environ.setdefault('UWSGI_WORKERS', str(worker_count))
        # If there is just one worker, cheaper is unnecessary.
        if worker_count > 1:
            os.environ.setdefault(
                'UWSGI_CHEAPER', str(min(worker_count, cpu_count)))

        # set process names
        os.environ.setdefault('UWSGI_AUTO_PROCNAME', 'true')
        os.environ.setdefault('UWSGI_PROCNAME_PREFIX_SPACED', '[uWSGI %s]' % django_project)
        # remove sockets/pidfile at exit
        os.environ.setdefault('UWSGI_VACUUM', 'true')
        # retrieve/set the PythonHome
        os.environ.setdefault('UWSGI_VIRTUALENV', sys.prefix)
        # add project to python path
        os.environ.setdefault('UWSGI_PP', root)

        os.environ.setdefault('UWSGI_POST_BUFFERING', '1048576')
        os.environ.setdefault('UWSGI_RELOAD_ON_RSS', '300')
        # increase buffer size a bit
        os.environ.setdefault('UWSGI_BUFFER_SIZE', '65535')
        # some additions required by newrelic
        os.environ.setdefault('UWSGI_ENABLE_THREADS', 'true')
        os.environ.setdefault('UWSGI_LAZY_APPS', 'true')
        os.environ.setdefault('UWSGI_SINGLE_INTERPRETER', 'true')
        os.environ.setdefault('UWSGI_AUTOLOAD', 'true')
        # enable the master process
        os.environ.setdefault('UWSGI_MASTER', 'true')

        os.environ.setdefault('UWSGI_NO_ORPHANS', 'true')
        os.environ.setdefault('UWSGI_MEMORY_REPORT', 'true')
        os.environ.setdefault('UWSGI_DISABLE_LOGGING', 'true')

        # set harakiri
        os.environ.setdefault('UWSGI_HARAKIRI', '60')
        os.environ.setdefault('UWSGI_HARAKIRI_VERBOSE', 'true')

        # set uid and gid
        os.environ.setdefault('UWSGI_UID', str(os.getuid()))
        os.environ.setdefault('UWSGI_GID', str(os.getgid()))
        # TODO: Figure out cache
        os.environ.setdefault('UWSGI_CACHE2', 'name=%s,items=20000,keysize=128,blocksize=4096' % django_project)
        if settings.DEBUG:
            if apps.is_installed('configurations'):
                os.environ.setdefault('DJANGO_CONFIGURATION', 'Development')
                import configurations
                configurations.setup()
            # map and serve static files
            os.environ.setdefault('UWSGI_STATIC_MAP', '%s=%s' % (settings.STATIC_URL, settings.STATIC_ROOT))
            os.environ.setdefault('UWSGI_PY_AUTORELOAD', '2')
        # run spooler for mail task
        if 'django_uwsgi' in settings.EMAIL_BACKEND:
            os.environ.setdefault('UWSGI_SPOOLER', '/tmp')
            os.environ.setdefault('UWSGI_SPOOLER_IMPORT', 'django_uwsgi.tasks')
        # exec the uwsgi binary
        if apps.ready:
            os.execvp('uwsgi', ('uwsgi',))


    def usage(self, subcomand):
        return r"""
  run this project on the uWSGI server
  http=PORT run the embedded http server on port PORT
  socket=ADDR bind the uwsgi server on address ADDR (this will disable the http server)
        """



import wagtail


if wagtail.VERSION >= (2, 0):
    from wagtail.admin.menu import MenuItem
    from wagtail.admin.site_summary import SummaryItem
    from wagtail.core import hooks
elif wagtail.VERSION < (2, 0):
    from wagtail.wagtailadmin.menu import MenuItem
    from wagtail.wagtailadmin.site_summary import SummaryItem
    from wagtail.wagtailcore import hooks



from django.core.mail.backends.base import BaseEmailBackend
from . import pickle


class EmailBackend(BaseEmailBackend):
    def send_messages(self, email_messages):
        num_sent = 0
        if not email_messages:
            return
        for email_message in email_messages:
            if self._send(email_message):
                num_sent += 1
        return num_sent

    def _send(self, email_message):
        from .tasks import send_mail
        send_mail.spool(body=pickle.dumps(email_message, 2))
        return True


from __future__ import unicode_literals
from django.core.mail import get_connection
from django.conf import settings

from .decorators import spool
from . import pickle


BACKEND = getattr(settings, 'UWSGI_EMAIL_BACKEND',
                  'django.core.mail.backends.smtp.EmailBackend')


@spool
def send_mail(arguments):
    conn = get_connection(backend=BACKEND)
    conn.send_messages([pickle.loads(arguments['body'])])



import django


if django.VERSION >= (2, 0):
    from django.urls import reverse_lazy, include, re_path as url
elif django.VERSION < (2, 0):
    from django.core.urlresolvers import reverse_lazy
    from django.conf.urls import include, url



import os
import time
from datetime import datetime
from django.utils.translation import ugettext_lazy as _
from . import uwsgi


def get_uwsgi_stats():
    uwsgi_stats = {}
    workers = uwsgi.workers()
    total_load = time.time() - uwsgi.started_on
    for w in workers:
        w['running_time'] = w['running_time'] / 1000
        w['avg_rt'] = w['avg_rt'] / 1000
        w['load'] = w['running_time'] / total_load / 10 / len(workers)
        w['last_spawn'] = datetime.fromtimestamp(w['last_spawn'])
    jobs = []
    if uwsgi.opt.get('spooler'):
        spooler_jobs = uwsgi.spooler_jobs()
        for j in spooler_jobs:
            jobs.append({'file': j, 'env': uwsgi.parsefile(str(j))})
    uwsgi_stats.update({
        'uwsgi': uwsgi,
        'stats': [
            ('loop', uwsgi.loop),
            ('masterpid', str(uwsgi.masterpid())),
            ('started_on', datetime.fromtimestamp(uwsgi.started_on)),
            ('now', datetime.now()),
            ('buffer_size', uwsgi.buffer_size),
            ('total_requests', uwsgi.total_requests()),
            ('numproc', uwsgi.numproc),
            ('cores', uwsgi.cores),
            ('cwd', os.getcwd()),
            ('logsize', uwsgi.logsize()),
            ('cache_exists', uwsgi.cache_exists),
            ('spooler_pid', uwsgi.spooler_pid() if uwsgi.opt.get('spooler') else _('disabled')),
            ('threads', _('enabled') if uwsgi.has_threads else _('disabled'))
        ],
        'workers': workers,
        'jobs': jobs
    })
    return uwsgi_stats



from django.utils.translation import ugettext_lazy as _

from . import uwsgi, urls
from .compat import include, url, reverse_lazy
from .wagtail_compat import hooks, MenuItem, SummaryItem


class UwsgiSummaryItem(SummaryItem):
    order = 800
    template = 'uwsgi/wagtail_dashboard_item.html'

    def get_context(self):
        workers = int(uwsgi.numproc) if uwsgi else 0
        return {'workers': workers}


@hooks.register('construct_homepage_summary_items')
def add_uwsgi_summary_item(request, items):
    items.append(UwsgiSummaryItem(request))


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^uwsgi/', include(urls)),
    ]


class UwsgiMenuItem(MenuItem):
    def is_shown(self, request):
        return request.user.is_staff


@hooks.register('register_settings_menu_item')
def register_uwsgi_menu_item():
    return UwsgiMenuItem(_('uWSGI Status'), reverse_lazy('uwsgi_index'), classnames='icon icon-cogs', order=800)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-17 20:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Vassal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Name')),
                ('name', models.CharField(db_column='name', editable=False, help_text='Name of uwsgi config file', max_length=255, verbose_name='Filename')),
                ('extension', models.CharField(choices=[('.ini', 'INI file'), ('.yml', 'YAML file'), ('.xml', 'XML file'), ('.json', 'JSON file')], default=('.ini', 'INI file'), max_length=4, verbose_name='Extension')),
                ('config', models.TextField(db_column='config', help_text='Config file blob', verbose_name='Config')),
                ('ts', models.FloatField(db_column='ts', editable=False, help_text='A number representing the modification time of this row in UNIX format', verbose_name='Unix Timestamp')),
                ('uid', models.PositiveSmallIntegerField(blank=True, db_column='uid', help_text='Required in Tyrant mode (secure multi-user hosting) mode only.', null=True, verbose_name='The UID of the vassal instance')),
                ('gid', models.PositiveSmallIntegerField(blank=True, db_column='gid', help_text='Required in Tyrant mode (secure multi-user hosting) mode only.', null=True, verbose_name='The GID of the vassal instance')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated', models.DateTimeField(auto_now=True, null=True, verbose_name='Updated')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled')),
            ],
            options={
                'verbose_name': "Emperor's Vassal",
                'verbose_name_plural': "Emperor's Vassals",
                'db_table': 'vassals',
            },
        ),
    ]



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from .models import Vassal


def enabled(modeladmin, request, queryset):
    queryset.update(enabled='1')
enabled.short_description = _("Enable selected Emperor's Vassals")


def disabled(modeladmin, request, queryset):
    queryset.update(enabled='0')
disabled.short_description = _("Disable selected Emperor's Vassals")


class VassalFields(object):
    list_display = ('title', 'extension', 'updated', 'created', 'enabled', 'ts')
    search_fields = ['title']
    list_filter = ('enabled', 'created', 'extension')


class VassalAdmin(VassalFields, admin.ModelAdmin):
    actions = [enabled, disabled]

admin.site.register(Vassal, VassalAdmin)
